class Result:

    def __init__(self, _res, _err):
        self._res = _res
        self._err = _err

    def is_ok(self):
        return self._err != None

    def err(self):
        return self._err

    def unwrap(self):
        return self._res

    @staticmethod
    def of(res, err):
        return Result(res, err)



class String:

    def __init__(self, _wrapped):
        self._wrapped = _wrapped

    @staticmethod
    def wrap(_wrapped):
        return String(_wrapped)

    def length(self):
        return len(self._wrapped)

    def __str__(self):
        return self._wrapped

    def compare_to(self, other):
        if not isinstance(other, String):
            return 1
        if self._wrapped < other._wrapped:
            return -1
        elif self._wrapped > other._wrapped:
            return 1
        return 0



from otterop.lang.string import String

class Array:
    def __init__(self, list, start, end):
        self._wrapped = list
        self._start = start
        self._end = end

    def get(self, i):
        return self._wrapped[self._start + i]

    def set(self, i, value):
        self._wrapped[self._start + i] = value

    def size(self):
        return self._end - self._start

    def slice(self, start, end):
        new_start = self._start + start
        new_end = self._start + end
        if new_start < self._start or new_start > self._end or new_end < new_start \
           or new_end > self._end:
             raise IndexError("slice arguments out of bounds")
        return Array(self._wrapped, new_start, new_end)

    @staticmethod
    def wrap(list):
        return Array(list, 0, len(list))

    @staticmethod
    def wrap_string(list):
        list = [ String.wrap(s) for s in list ]
        return Array.wrap(list)


from otterop.lang.string import String

class Error:

    def __init__(self, code, message):
        self.code = code
        self.message = message

    def code(self):
        return self.code

    def message(self):
        return self.message
